import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:image_picker/image_picker.dart';

void main() {
  runApp(Exo10());
}

class Exo10 extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: MyHomePage(),
    );
  }
}

class MyHomePage extends StatefulWidget {
  @override
  _MyHomePageState createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  File? _image;
  final picker = ImagePicker();
  List<Widget> tiles = <Widget>[];
  int colNb = 3;
  int emptyTileIndex = 0;

  @override
  void initState() {
    super.initState();
    tiles = buildImage(Image.asset('assets/placeholder_image.jpg'));
  }

  List<Widget> buildImage(Image image) {
    int imageCount = 0;
    List<Widget> res = <Widget>[];

    emptyTileIndex = colNb * colNb - 1;

    for (int i = 0; i < colNb; i++) {
      for (int j = 0; j < colNb; j++) {
        int indexcopy = imageCount;

        Tile tile = Tile(
          image: image,
          alignment: Alignment(
            -1 + 2 * j.toDouble() / (colNb.toDouble() - 1),
            -1 + 2 * i.toDouble() / (colNb.toDouble() - 1),
          ),
        );

        Key k = UniqueKey();

        res.add(
          Flexible(
            key: k,
            child: Container(
              margin: EdgeInsets.all(2),
              child: InkWell(
                child: tile.croppedImageTile(1 / colNb),
                onTap: () {
                  setState(() {
                    swapTiles(k);
                  });
                },
              ),
            ),
          ),
        );

        imageCount++;
      }
    }

    res.shuffle();
    res[emptyTileIndex] = Flexible(child: Container(margin: EdgeInsets.all(2)));

    return res;
  }

  void swapTiles(Key k) {
    Widget t = tiles.singleWhere((element) => element.key == k);
    int i = tiles.indexOf(t);

    // If tile is neighbor of empty tile => move
    if (i + 1 == emptyTileIndex && i % colNb != colNb - 1 ||
        i - 1 == emptyTileIndex && i % colNb != 0 ||
        i - colNb == emptyTileIndex ||
        i + colNb == emptyTileIndex) {
      tiles[emptyTileIndex] = t;
      tiles[i] = Flexible(child: Container(margin: EdgeInsets.all(2)));
      emptyTileIndex = i;
    }
  }

  Future getImage() async {
    final pickedFile = await picker.pickImage(source: ImageSource.gallery);

    setState(() {
      if (pickedFile != null) {
        _image = File(pickedFile.path);
        tiles = buildImage(Image.file(_image!));
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('Image Picker Example'),
      ),
      body: Column(
        children: [
          TextButton(
            child: Text('Select Image'),
            onPressed: getImage,
          ),
          Slider(
            value: colNb.toDouble(),
            min: 2,
            max: 6,
            divisions: 5,
            label: colNb.toString(),
            onChanged: (double value) {
              setState(() {
                colNb = value.toInt();
                tiles = buildImage(_image == null ? Image.asset('assets/placeholder_image.jpg') : Image.file(_image!));
              });
            },
          ),
          Expanded(
            child: Padding(
              padding: const EdgeInsets.all(8.0),
              child: GridView.builder(
                gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                  crossAxisCount: colNb,
                ),
                itemCount: colNb * colNb,
                itemBuilder: (BuildContext context, int index) {
                  return tiles.elementAt(index);
                },
              ),
            ),
          ),
        ],
      ),
    );
  }
}

class Tile {
  Image image;
  Alignment alignment;

  Tile({required this.image, required this.alignment});

  Widget croppedImageTile(double widthFactor) {
    return FittedBox(
      fit: BoxFit.fill,
      child: ClipRect(
        child: Container(
          child: Align(
            alignment: alignment,
            widthFactor: widthFactor,
            heightFactor: widthFactor,
            child: image,
          ),
        ),
      ),
    );
  }
}
