import 'package:flutter/material.dart';
import 'dart:math' as math;

class Tile {
  Image image;
  Alignment alignment;

  Tile({required this.image, required this.alignment});

  Widget croppedImageTile(double widthFactor) {

    return FittedBox(
      fit: BoxFit.fill,
      child: ClipRect(
        child: Container(
          child: Align(
            alignment: this.alignment,
            widthFactor: widthFactor,
            heightFactor: widthFactor,
            child: image,
          ),
        ),
      ),
    );
  }
}

class Exo6 extends StatefulWidget {
  const Exo6({super.key});

  @override
  State<Exo6> createState() => _Exo6();
}

class _Exo6 extends State<Exo6> {
  var image = Image.network('https://picsum.photos/512');
  List<Widget> tiles = <Widget>[];
  double? size = 300;
  int colNb = 3;
  int emptyTileIndex = 0;

  @override
  void initState(){
    super.initState();

    image = Image.network('https://picsum.photos/512');
    tiles = buildImage();
  }

  List<Widget> buildImage(){
    int imageCount = 0;
    List<Widget> res = <Widget>[];

    emptyTileIndex = math.Random().nextInt(colNb * colNb);

    for(int i = 0; i < colNb; i++){
      for(int j = 0; j < colNb; j++){
        int indexcopy = imageCount;

        Tile tile = Tile(image: image, alignment: 
          Alignment(-1 + 2 * j.toDouble() / (colNb.toDouble() - 1),
                    -1 + 2 * i.toDouble() / (colNb.toDouble() - 1))
        );

        Key k = UniqueKey();

        res.add(
          Flexible(key: k, child: Container(
            margin: EdgeInsets.all(2),
            child: InkWell(
              child: tile.croppedImageTile(1 / colNb),
              onTap: () {
                setState(() {
                  swapTiles(k);
                });
              }
            )
          ))
        );

        imageCount++;
      }
    }

    res.shuffle();
    res[emptyTileIndex] = Flexible(child: Container(margin: EdgeInsets.all(2)));

    return res;
  }

  void swapTiles(Key k){
    Widget t = tiles.singleWhere((element) => element.key == k);
    int i = tiles.indexOf(t);

    // If tile is neighbor of empty tile => move
    if(
      i+1 == emptyTileIndex && i%colNb != colNb-1 ||
      i-1 == emptyTileIndex && i%colNb != 0 ||
      i-colNb == emptyTileIndex ||
      i+colNb == emptyTileIndex
    ){

      tiles[emptyTileIndex] = t;
      tiles[i] = Flexible(child: Container(margin: EdgeInsets.all(2)));
      emptyTileIndex = i;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('Display a Tile as a Cropped Image'),
        centerTitle: true,
      ), 
      body: Column(children: [
        Container(
          margin: const EdgeInsets.all(10.0),
          child: SizedBox(
            height: MediaQuery.of(context).size.height * 0.7,
            width: MediaQuery.of(context).size.height * 0.7,
            child: GridView.builder(
              gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                crossAxisCount: colNb,
              ),
              itemCount: colNb * colNb,
              itemBuilder: (BuildContext context, int index) {
                return tiles.elementAt(index);
              }
            )
          )
        ),
        Slider(
          value: colNb.toDouble(),
          min: 2,
          max: 6,
          onChanged: (double value) {
            if(value.toInt() != colNb){
              setState(() {
                colNb = value.toInt();
                tiles = buildImage();
              });
            }
          }
        ),
      ]),
    );
  }
}
